@@ -1,7 +1,9 @@
 from typing import Optional, Union, List
 from datetime import datetime
 from concurrent.futures import ThreadPoolExecutor
+import itertools
 
+import numpy as np
 import pandas as pd
 
 from .. import _credentials
@@ -406,21 +408,32 @@ def update_batch_values(
 
     data.index = data.index.get_level_values(batch_id_index_name)
 
-    # Send each column
-    def send_row(row_tuple: tuple):
-        row = row_tuple[1]
-        for col in row.index:
-            endpoints.batches.update_batch_value(
-                batch_type_id=batch_type_id,
-                batch_id=str(row.name),
-                data_id=col,
-                value=row[col],
-                unit_id=unit_ids.get(col, None),
-                api_credentials=api_credentials,
-            )
+    # Send each individual value
+    def send_value(batch_value_tuple: tuple):
+        batch_id = batch_value_tuple[0]
+        data_id = batch_value_tuple[1]
+        value = batch_value_tuple[2]
+        endpoints.batches.update_batch_value(
+            batch_type_id=batch_type_id,
+            batch_id=batch_id,
+            data_id=data_id,
+            value=value,
+            unit_id=unit_ids.get(data_id, None),
+            api_credentials=api_credentials,
+        )
+
+    # Build the iterator over individual batch value tuples (batch_id, data_id, value)
+    batch_values = list(
+        itertools.chain.from_iterable(
+            [
+                [(idx,) + i for i in r.iteritems() if not np.isnan(i[1])]
+                for idx, r in data.iterrows()
+            ]
+        )
+    )
 
     with ThreadPoolExecutor() as pool:
-        pool.map(send_row, data.iterrows())
+        pool.map(send_value, batch_values)
 
 
 def update_batch_feature_values(
@@ -437,17 +450,28 @@ def update_batch_feature_values(
 
     data.index = data.index.get_level_values(batch_id_index_name)
 
-    # Send each column
-    def send_row(row_tuple: tuple):
-        row = row_tuple[1]
-        for col in row.index:
-            endpoints.batches.update_batch_feature_value(
-                batch_type_id=batch_type_id,
-                batch_id=str(row.name),
-                feature_id=col,
-                value=row[col],
-                api_credentials=api_credentials,
-            )
+    # Send each individual value
+    def send_value(batch_value_tuple: tuple):
+        batch_id = batch_value_tuple[0]
+        feature_id = batch_value_tuple[1]
+        value = batch_value_tuple[2]
+        endpoints.batches.update_batch_feature_value(
+            batch_type_id=batch_type_id,
+            batch_id=batch_id,
+            feature_id=feature_id,
+            value=value,
+            api_credentials=api_credentials,
+        )
+
+    # Build the iterator over individual batch value tuples (batch_id, data_id, value)
+    batch_values = list(
+        itertools.chain.from_iterable(
+            [
+                [(idx,) + i for i in r.iteritems() if not np.isnan(i[1])]
+                for idx, r in data.iterrows()
+            ]
+        )
+    )
 
     with ThreadPoolExecutor() as pool:
-        pool.map(send_row, data.iterrows())
+        pool.map(send_value, batch_values)
